@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2012-2014 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2012-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,22 +82,22 @@ int DHT::Do()
    }
    if(sent_req_expire_scan.Stopped()) {
       for(const Request *r=sent_req.each_begin(); r; r=sent_req.each_next()) {
-	 if(r->Expired()) {
-	    LogError(4,"DHT request %s to %s timed out",r->data->lookup_str("q").get(),r->addr.to_string());
-	    Node *n=nodes.lookup(r->GetNodeId());
-	    const xstring& target=r->GetSearchTarget();
-	    if(target) {
-	       Search *s=search.lookup(target);
-	       // if we have lost a search request and have not received any
-	       // reply yet - try to restart the search with good nodes.
-	       if(s && !s->best_node_id)
-		  RestartSearch(s);
-	    }
-	    sent_req.remove(sent_req.each_key());
-	    if(n) {
-	       n->LostPing();
-	       LogNote(4,"DHT node %s has lost %d packets",n->GetName(),n->ping_lost_count);
-	    }
+	 if(!r->Expired())
+	    continue;
+	 Ref<Request> rr(sent_req.borrow(sent_req.each_key()));
+	 LogError(4,"DHT request %s to %s timed out",r->data->lookup_str("q").get(),r->addr.to_string());
+	 Node *n=nodes.lookup(r->GetNodeId());
+	 if(n) {
+	    n->LostPing();
+	    LogNote(4,"DHT node %s has lost %d packets",n->GetName(),n->ping_lost_count);
+	 }
+	 const xstring& target=r->GetSearchTarget();
+	 if(target) {
+	    Search *s=search.lookup(target);
+	    // if we have lost a search request and have not received any
+	    // reply yet - try to restart the search with good nodes.
+	    if(s && !s->best_node_id)
+	       RestartSearch(s);
 	 }
       }
       sent_req_expire_scan.Reset();
@@ -140,7 +140,7 @@ int DHT::Do()
 	    xarray<Node*>& nodes=routes[i]->nodes;
 	    int q_num=PingQuestionable(nodes,nodes.count()-K);
 	    if(nodes.count()>K+q_num)
-	       routes[i]->RemoveNode(K); // too may candidates, trim one
+	       routes[i]->RemoveNode(K); // too many candidates, trim one
 	 }
       }
       // remove bad peers
@@ -169,14 +169,16 @@ int DHT::Do()
    if(refresh_timer.Stopped()) {
       for(int i=0; i<routes.count(); i++) {
 	 if(!routes[i]->IsFresh()) {
-	    LogNote(9,"refreshing route bucket %d",i);
+	    LogNote(9,"refreshing route bucket %d (prefix=%s)",i,routes[i]->to_string());
 	    // make random id in the range
 	    int bytes=routes[i]->prefix_bits/8;
 	    int bits=routes[i]->prefix_bits%8;
 	    xstring random_id(routes[i]->prefix.get(),bytes+(bits>0));
-	    unsigned mask=(1<<(8-bits))-1;
-	    if(bits>0)
+	    if(bits>0) {
+	       unsigned mask=(1<<(8-bits))-1;
+	       assert(!(random_id[bytes]&mask));
 	       random_id.get_non_const()[bytes]|=(random()/13)&mask;
+	    }
 	    while(random_id.length()<20)
 	       random_id.append(char(random()/13));
 	    StartSearch(new Search(random_id));
@@ -208,7 +210,7 @@ int DHT::Do()
       m=MOVED;
    }
 
-   if(send_queue.count()>0 && MaySendMessage()) {
+   while(send_queue.count()>0 && MaySendMessage()) {
       SendMessage(send_queue.next().borrow());
       m=MOVED;
    }
@@ -378,18 +380,18 @@ void DHT::HandlePacket(BeNode *p,const sockaddr_u& src)
    LogRecv(4,xstring::format("received DHT %s from %s %s",MessageType(p),
       src.to_string(),p->Format1()));
    int pkt_len=p->str.length();
-   if(rate_limit.BytesAllowedToGet()<pkt_len) {
-      LogError(9,"dropping incoming message (rate limit exceeded)");
-      return;
-   }
-   rate_limit.BytesGot(pkt_len);
    const xstring& t=p->lookup_str("t");
    if(!t)
       return;
    const xstring& y=p->lookup_str("y");
    if(!y)
       return;
-   if(y.eq("q")) {
+   if(y.eq("q")) { // query
+      if(rate_limit.BytesAllowedToGet()<pkt_len) {
+	 LogError(9,"dropping incoming message (rate limit exceeded)");
+	 return;
+      }
+      rate_limit.BytesGot(pkt_len);
       const xstring& q=p->lookup_str("q");
       if(!q)
 	 return;
@@ -524,20 +526,30 @@ void DHT::HandlePacket(BeNode *p,const sockaddr_u& src)
    }
 
    const xstring& q=req->data->lookup_str("q");
-   if(y.eq("r")) {
+   if(y.eq("r")) { // reply
       BeNode *r=p->lookup("r",BeNode::BE_DICT);
       if(!r)
 	 return;
       const xstring& id=r->lookup_str("id");
       if(id.length()!=20)
 	 return;
+
+      Node *node=FoundNode(id,src,true);
+      if(!node)
+	 return;
+
       const sockaddr_compact& ip=sockaddr_compact::cast(r->lookup_str("ip"));
       if(ip && !ValidNodeId(node_id,ip)) {
-	 if(!ip_voted.lookup(src.compact_addr())) {
-	    sockaddr_u reported_ip(ip);
-	    LogNote(2,"%s reported our IP as %s",src.to_string(),reported_ip.address());
+	 const xstring &src_ip=xstring::get_tmp(src.address());
+	 if(src_ip.eq(ip.address())) {
+	    LogError(2,"%s incorrectly reported our IP as %s",src.to_string(),ip.address());
+	    black_list.Add(src,"1d");
+	    return;
+	 } else if(!ip_voted.lookup(src_ip)) {
+	    ip_voted.add(src_ip,true);
 	    unsigned& votes=ip_votes.lookup_Lv(ip);
 	    votes++;
+	    LogNote(2,"%s reported our IP as %s (votes=%u)",src.to_string(),ip.address(),votes);
 	    if(votes>=4) {
 	       // we have incorrect node_id, restart with correct one.
 	       MakeNodeId(node_id,ip);
@@ -546,7 +558,7 @@ void DHT::HandlePacket(BeNode *p,const sockaddr_u& src)
 	    }
 	 }
       }
-      FoundNode(id,src,true);
+
       if(q.eq("get_peers")) {
 	 const xstring& info_hash=req->GetSearchTarget();
 	 Torrent *torrent=Torrent::FindTorrent(info_hash);
@@ -609,7 +621,9 @@ void DHT::HandlePacket(BeNode *p,const sockaddr_u& src)
 	       a.set_compact(data+20,6);
 	       data+=26;
 	       len-=26;
-	       FoundNode(id,a,false,s);
+	       Node *new_node=FoundNode(id,a,false,s);
+	       if(new_node)
+		  new_node->SetOrigin(node);
 	    }
 	 }
 #if INET6
@@ -624,12 +638,14 @@ void DHT::HandlePacket(BeNode *p,const sockaddr_u& src)
 	       a.set_compact(data+20,18);
 	       data+=38;
 	       len-=38;
-	       FoundNode(id,a,false,s);
+	       Node *new_node=FoundNode(id,a,false,s);
+	       if(new_node)
+		  new_node->SetOrigin(node);
 	    }
 	 }
 #endif //INET6
       }
-   } else if(y.eq("e")) {
+   } else if(y.eq("e")) { // error
       int code=0;
       const char *msg="unknown";
       BeNode *e=p->lookup("e",BeNode::BE_LIST);
@@ -657,6 +673,7 @@ bool DHT::Search::IsFeasible(const xstring &id) const
    }
    return false;
 }
+
 void DHT::Search::ContinueOn(DHT *d,const Node *n)
 {
    if(searched.exists(n->id)) {
@@ -687,26 +704,40 @@ void DHT::Search::ContinueOn(DHT *d,const Node *n)
    searched.add(n->id,true);
    search_timer.Reset();
 }
+
 void DHT::StartSearch(Search *s)
 {
    LogNote(9,"starting search for %s",s->target_id.hexdump());
    xarray<Node*> n;
    FindNodes(s->target_id,n,K,true);
-   if(n.count()==0) {
-      LogError(2,"no good nodes found in the routing table");
+   if(n.count()<=K/2) {
+      LogNote(2,"too few good nodes found in the routing table");
       FindNodes(s->target_id,n,K,false);
+      if(n.count()==0)
+	 LogError(1,"no nodes found in the routing table");
    }
    for(int i=0; i<n.count(); i++)
       s->ContinueOn(this,n[i]);
    search.add(s->target_id,s);
 }
+
 void DHT::RestartSearch(Search *s)
 {
    xarray<Node*> n;
-   FindNodes(s->target_id,n,K,true);
+   FindNodes(s->target_id,n,K,true,&s->searched);
    for(int i=0; i<n.count(); i++)
       s->ContinueOn(this,n[i]);
 }
+
+DHT::Node *DHT::GetOrigin(const Node *n) {
+   if(!n->origin_id)
+      return 0;
+   Node *origin=nodes.lookup(n->origin_id);
+   if(!origin || origin==n)
+      return 0;
+   return origin;
+}
+
 DHT::Node *DHT::FoundNode(const xstring& id,const sockaddr_u& a,bool responded,Search *s)
 {
    if(a.port()==0 || a.is_private() || a.is_reserved() || a.is_multicast()) {
@@ -714,18 +745,32 @@ DHT::Node *DHT::FoundNode(const xstring& id,const sockaddr_u& a,bool responded,S
       return 0;
    }
 
-   if(a.family()!=af) {
-      assert(!responded); // we should not get replies from different AF
-      const SMTaskRef<DHT>& d=Torrent::GetDHT(a);
-      d->Enter();
-      d->FoundNode(id,a,false);
-      d->Leave();
+   if(a.family()!=af)
+      return 0;
+
+   if(id.eq(node_id)) {
+      LogNote(9,"node %s has our own id",a.to_string());
       return 0;
    }
+
+   if(black_list.Listed(a)) {
+      LogNote(9,"node %s is blacklisted",a.to_string());
+      return 0;
+   }
+
    Node *n=nodes.lookup(id);
    if(!n) {
       n=node_by_addr.lookup(a.compact());
       if(n) {
+	 if(!responded) {
+	    // change node id only by a message from the node.
+	    return 0;
+	 }
+	 if(n->id_change_count>0) {
+	    LogError(9,"%s changes node id again",n->addr.to_string());
+	    BlackListNode(n,"1d");
+	    return 0;
+	 }
 	 ChangeNodeId(n,id);
       } else {
 	 n=new Node(id,a);
@@ -738,6 +783,9 @@ DHT::Node *DHT::FoundNode(const xstring& id,const sockaddr_u& a,bool responded,S
    if(responded) {
       n->responded=true;
       n->ResetLostPing();
+      Node *origin=GetOrigin(n);
+      if(origin)
+	 origin->bad_node_count/=2;
    }
    if(n->responded)
       n->SetGood();
@@ -750,6 +798,12 @@ DHT::Node *DHT::FoundNode(const xstring& id,const sockaddr_u& a,bool responded,S
 }
 void DHT::RemoveNode(Node *n)
 {
+   Node *origin=GetOrigin(n);
+   if(origin && !n->responded && n->ping_lost_count>1) {
+      origin->bad_node_count++;
+      if(origin->bad_node_count>2*K)
+	 BlackListNode(origin,"1h");
+   }
    RemoveRoute(n);
    node_by_addr.remove(n->addr.compact());
    nodes.remove(n->id);
@@ -758,6 +812,7 @@ void DHT::ChangeNodeId(Node *n,const xstring& new_node_id)
 {
    LogNote(1,"node_id changed for %s, old_node_id=%s, new_node_id=%s",
       n->GetName(),n->id.hexdump(),new_node_id.hexdump());
+   n->id_change_count++;
 
    // change node_id in the in-flight requests
    for(Request *r=sent_req.each_begin(); r; r=sent_req.each_next()) {
@@ -771,6 +826,20 @@ void DHT::ChangeNodeId(Node *n,const xstring& new_node_id)
    nodes.add(n->id,n);
    AddRoute(n);
 }
+void DHT::BlackListNode(Node *n,const char *timeout)
+{
+   black_list.Add(n->addr,timeout);
+   for(int i=0; i<send_queue.count(); i++) {
+      if(send_queue[i]->node_id.eq(n->id))
+	 send_queue.remove(i);
+   }
+   for(const Request *r=sent_req.each_begin(); r; r=sent_req.each_next()) {
+      if(r->GetNodeId().eq(n->id))
+	 sent_req.remove(sent_req.each_key());
+   }
+   RemoveNode(n);
+}
+
 void DHT::AddNode(Node *n)
 {
    assert(n->id.length()==20);
@@ -782,15 +851,15 @@ void DHT::AddNode(Node *n)
 
    AddRoute(n);
 
-   if(nodes.count()==1 && search.count()==0)
+   if(nodes.count()==1 && search.count()==0 && !state_io)
       Bootstrap();
 }
-int DHT::FindRoute(const xstring& id,int i)
+int DHT::FindRoute(const xstring& id,int i,int skew)
 {
    // routes are ordered by prefix length decreasing
    // the first route bucket always matches our node_id
    for( ; i<routes.count(); i++) {
-      if(routes[i]->PrefixMatch(id)) {
+      if(routes[i]->PrefixMatch(id,skew)) {
 	 return i;
       }
    }
@@ -805,13 +874,13 @@ void DHT::RemoveRoute(Node *n)
 }
 void DHT::AddRoute(Node *n)
 {
+try_again:
    int i=FindRoute(n->id);
    if(i==-1) {
       assert(routes.count()==0);
       routes.append(new RouteBucket(0,xstring::null));
       i=0;
    }
-try_again:
    const Ref<RouteBucket>& r=routes[i];
    xarray<Node*> &nodes=r->nodes;
    // check if the node is already in the bucket
@@ -860,45 +929,17 @@ try_again:
       }
    }
 
-   if(nodes.count()>=K && i==0 && r->prefix_bits<160) {
-      // split the bucket
-      int bits=r->prefix_bits;
-      size_t byte=bits/8;
-      unsigned mask = 1<<(7-bits%8);
-      if(r->prefix.length()<=byte)
-	 r->prefix.append('\0');
-      xstring p1(r->prefix.copy());
-      xstring p2(r->prefix.copy());
-      p1.get_non_const()[byte]&=~mask;
-      p2.get_non_const()[byte]|=mask;
-      RouteBucket *b1=new RouteBucket(bits+1,p1);
-      RouteBucket *b2=new RouteBucket(bits+1,p2);
-      for(int j=0; j<nodes.count(); j++) {
-	 if(nodes[j]->id[byte]&mask)
-	    b2->nodes.append(nodes[j]);
-	 else
-	    b1->nodes.append(nodes[j]);
-      }
-      if(node_id[byte]&mask) {
-	 routes[0]=b2;
-	 routes.insert(b1,1);
-	 i=(n->id[byte]&mask)?0:1;
-      } else {
-	 routes[0]=b1;
-	 routes.insert(b2,1);
-	 i=(n->id[byte]&mask)?1:0;
-      }
-      LogNote(9,"splitted route bucket 0");
-      LogNote(9,"new route[0] prefix=%s",routes[0]->to_string());
-      LogNote(9,"new route[1] prefix=%s",routes[1]->to_string());
-      assert(routes[0]->PrefixMatch(node_id));
+   if(state_io && i==0 && nodes.count()>=K && SplitRoute0())
       goto try_again;
-   }
+
    if(nodes.count()>=K) {
       int q_num=PingQuestionable(nodes,nodes.count()-K+1);
       // check if we have already candidates for the questionable nodes
       if(nodes.count()>=K+q_num) {
-	 LogNote(9,"skipping node %s (too many in route bucket %d)",n->GetName(),i);
+	 if(i==0 && SplitRoute0())
+	    goto try_again;
+	 else
+	    LogNote(9,"skipping node %s, route bucket %d (prefix=%s) has %d nodes",n->GetName(),i,r->to_string(),nodes.count());
 	 return;
       }
    }
@@ -907,6 +948,47 @@ try_again:
    n->in_routes=true;
    nodes.append(n);
 }
+bool DHT::SplitRoute0()
+{
+   const Ref<RouteBucket>& r=routes[0];
+   xarray<Node*> &nodes=r->nodes;
+
+   if(nodes.count()<K || r->prefix_bits>=160)
+      return false;
+   if(routes.count()>1 && !routes[1]->HasGoodNodes() && !state_io)
+      return false;
+
+   LogNote(9,"splitting route bucket 0, nodes=%d",nodes.count());
+   int bits=r->prefix_bits;
+   size_t byte=bits/8;
+   unsigned mask = 1<<(7-bits%8);
+   if(r->prefix.length()<=byte)
+      r->prefix.append('\0');
+   xstring p1(r->prefix.copy());
+   xstring p2(r->prefix.copy());
+   // new bit in p1 is already cleared.
+   p2.get_non_const()[byte]|=mask; // set new bit in p2
+   RouteBucket *b1=new RouteBucket(bits+1,p1);
+   RouteBucket *b2=new RouteBucket(bits+1,p2);
+   // distribute the nodes between two buckets
+   for(int j=0; j<nodes.count(); j++) {
+      if(nodes[j]->id[byte]&mask)
+	 b2->nodes.append(nodes[j]);
+      else
+	 b1->nodes.append(nodes[j]);
+   }
+   if(node_id[byte]&mask) {
+      routes[0]=b2;
+      routes.insert(b1,1);
+   } else {
+      routes[0]=b1;
+      routes.insert(b2,1);
+   }
+   LogNote(9,"new route[0] prefix=%s nodes=%d",routes[0]->to_string(),routes[0]->nodes.count());
+   LogNote(9,"new route[1] prefix=%s nodes=%d",routes[1]->to_string(),routes[1]->nodes.count());
+   assert(routes[0]->PrefixMatch(node_id));
+   return true;
+}
 int DHT::PingQuestionable(const xarray<Node*>& nodes,int limit)
 {
    int q_num=0;
@@ -936,8 +1018,12 @@ void DHT::RouteBucket::RemoveNode(int i)
    nodes[i]->in_routes=false;
    nodes.remove(i);
 }
-bool DHT::RouteBucket::PrefixMatch(const xstring& id) const
+bool DHT::RouteBucket::PrefixMatch(const xstring& id,int skew) const
 {
+   assert(skew>=0);
+   int prefix_bits=this->prefix_bits-skew;
+   if(prefix_bits<=0)
+      return true;
    int bytes=prefix_bits/8;
    int bits=prefix_bits%8;
    unsigned mask=~((1<<(8-bits))-1);
@@ -957,23 +1043,23 @@ const char *DHT::RouteBucket::to_string() const
    return buf;
 }
 
-void DHT::FindNodes(const xstring& target_id,xarray<Node*> &a,int max_count,bool only_good)
+void DHT::FindNodes(const xstring& target_id,xarray<Node*> &a,int max_count,bool only_good,const xmap<bool> *exclude)
 {
    a.truncate();
-   int i=0;
-   while(a.count()<max_count && i<routes.count()) {
-      i=FindRoute(target_id,i);
-      if(i==-1)
-	 return;
+   for(int skew=0; skew<160; skew++) {
+      int i=FindRoute(target_id,0,skew);
+      if(i<0)
+	 continue;
       const xarray<Node*> &nodes=routes[i]->nodes;
-      int limit=max_count-a.count();
-      for(int j=0; j<nodes.count() && limit>0; j++) {
-	 if(!nodes[j]->IsBad() && (!only_good || nodes[j]->IsGood())) {
+      for(int j=0; j<nodes.count(); j++) {
+	 if(!nodes[j]->IsBad() && (!only_good || nodes[j]->IsGood())
+	 && nodes[j]->ping_lost_count<2 && a.search(nodes[j])==-1
+	 && (!exclude || !exclude->exists(nodes[j]->id))) {
 	    a.append(nodes[j]);
-	    limit--;
+	    if(a.count()>=max_count)
+	       return;
 	 }
       }
-      i++;
    }
 }
 void DHT::AddPeer(const xstring& info_hash,const sockaddr_compact& a,bool seed)
@@ -1062,6 +1148,7 @@ void DHT::Restart()
 }
 void DHT::Save(const SMTaskRef<IOBuffer>& buf)
 {
+   Enter(this);
    xmap_p<BeNode> state;
    state.add("id",new BeNode(node_id));
    int count=0;
@@ -1084,6 +1171,7 @@ void DHT::Save(const SMTaskRef<IOBuffer>& buf)
       LogNote(9,"route bucket %d: nodes count=%d prefix=%s",i,
 	 (int)r->nodes.count(),r->to_string());
    }
+   Leave();
 }
 void DHT::Load(const SMTaskRef<IOBuffer>& buf)
 {
@@ -1110,16 +1198,10 @@ void DHT::Load(const SMTaskRef<IOBuffer>& buf)
       data+=node_len;
       len-=node_len;
       FoundNode(id,a,false);
-      // loaded node is not good by default (learned long ago).
-      Node *n=DHT::nodes.lookup(id);
-      if(n) {
-	 n->good_timer.Stop();
-	 n->ping_timer.Stop();
-      }
    }
    // refresh routes after loading
    for(int i=0; i<routes.count(); i++)
-      routes[i]->fresh_timer.StopDelayed(i+3);
+      routes[i]->fresh_timer.StopDelayed(i*15+3);
 }
 void DHT::Save()
 {
@@ -1147,3 +1229,24 @@ void DHT::Reconfig(const char *name)
 {
    rate_limit.Reconfig(name,"DHT");
 }
+
+bool DHT::BlackList::Listed(const sockaddr_u& addr)
+{
+   const xstring &key=addr.to_xstring();
+   Timer *e=bl.lookup(key);
+   if(!e)
+      return false;
+   if(e->Stopped()) {
+      LogNote(4,"black-delisting node %s\n",key.get());
+      bl.remove(key);
+      return false;
+   }
+   return true;
+}
+void DHT::BlackList::Add(const sockaddr_u &a,const char *t)
+{
+   if(Listed(a))
+      return;
+   LogNote(4,"black-listing node %s (%s)\n",a.to_string(),t);
+   bl.add(a.to_xstring(),new Timer(TimeIntervalR(t)));
+}
