@@ -203,8 +203,8 @@ public:
    const char  *GetUser() const { return user; }
    const char  *GetPassword() const { return pass; }
    const char  *GetPort() const { return portname; }
-   const char  *GetConnectURL(int flags=0) const;
-   const char  *GetFileURL(const char *file,int flags=0) const;
+   const xstring& GetConnectURL(int flags=0) const;
+   const xstring& GetFileURL(const char *file,int flags=0) const;
    enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4,NO_USER=8 };
    const char *GetLastDisconnectCause() const { return last_disconnect_cause; }
 
