@@ -497,7 +497,6 @@ void FileCopy::SetError(const char *str)
 {
    error_text.set(str);
    get=0;
-   put=0;
 }
 
 void FileCopy::LineBuffered(int s)
@@ -828,6 +827,7 @@ FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
    removing=false;
    file_removed=false;
    temp_file=false;
+   do_mkdir=false;
    use_cache=true;
    write_allowed=true;
    done=false;
@@ -843,7 +843,18 @@ int FileCopyPeerFA::Do()
    int m=STALL;
    int res;
 
-   if(session->OpenMode()==FA::RENAME)
+   if(session->OpenMode()==FA::MAKE_DIR)
+   {
+      // doing mkdir
+      int res=session->Done();
+      if(res==FA::IN_PROGRESS)
+	 return m;
+      if(res<0)
+	 debug((3,"mkdir failed: %s\n",session->StrError(res)));
+      session->Close();
+      m=MOVED;
+   }
+   else if(session->OpenMode()==FA::RENAME)
    {
       int res=session->Done();
       if(res==FA::IN_PROGRESS)
@@ -859,6 +870,14 @@ int FileCopyPeerFA::Do()
       return MOVED;
    }
 
+   if(do_mkdir) {
+      // do mkdir just once
+      do_mkdir=false;
+      assert(!session->IsOpen());
+      session->Mkdir(dirname(file),true);
+      return MOVED;
+   }
+
    if(removing)
    {
       res=session->Done();
@@ -1258,7 +1277,7 @@ int FileCopyPeerFA::Get_LL(int len)
    }
    else if(res==0)
    {
-      debug((10,"copy-peer: EOF on %s\n",session->GetFileURL(session->GetFile())));
+      debug((10,"copy-peer: EOF on %s\n",session->GetFileURL(session->GetFile()).get()));
       eof=true;
       FileAccess::cache->Add(session,file,FAmode,FA::OK,this);
       SetSuggestedFileName(session->GetSuggestedFileName());
@@ -1526,6 +1545,11 @@ int FileCopyPeerFDStream::Do()
    if(Done() || Error())
       return m;
 
+   if(do_mkdir) {
+      do_mkdir=false;
+      create_directories(dirname(stream->full_name).get_non_const());
+   }
+
    if(verify)
    {
       if(verify->Error())
@@ -1988,6 +2012,7 @@ void FileVerificator::InitVerify(const char *f)
       return;
    ArgV *args=new ArgV(ResMgr::Query("xfer:verify-command",0));
    args->Append(f);
+   Log::global->Format(9,"running %s %s\n",args->a0(),f);
    verify_process=new InputFilter(args);
    verify_process->StderrToStdout();
    verify_buffer=new IOBufferFDStream(verify_process.Cast<FDStream>(),IOBuffer::GET);
