@@ -55,6 +55,7 @@ protected:
    bool removing;
    bool file_removed;
    bool temp_file;
+   bool do_mkdir;
 
    bool done;
 
@@ -91,6 +92,7 @@ public:
    void DontCopyDate() { do_set_date=false; }
    void DontVerify() { do_verify=false; }
    bool NeedDate() { return do_set_date; }
+   void MakeTargetDir() { do_mkdir=true; }
 
    void SetRange(const off_t s,const off_t lim);
 
@@ -174,6 +176,9 @@ private:
    int put_buf;
    off_t put_eof_pos;
 
+   off_t high_watermark;
+   Timer high_watermark_timeout;
+
    Time start_time;
    Time end_time;
 
@@ -255,6 +260,8 @@ public:
 
    void LogTransfer();
    static Ref<Log> transfer_log;
+
+   static const char *TempFileName(const char *file);
 };
 
 class FileVerificator : public SMTask
@@ -339,7 +346,7 @@ public:
 
    const char *GetDescriptionForLog()
       {
-	 return orig_url ? orig_url.get() : session->GetFileURL(file);
+	 return orig_url ? orig_url : session->GetFileURL(file);
       }
    const char *GetURL() { return GetDescriptionForLog(); }
    FileCopyPeer *Clone();
