@@ -121,9 +121,23 @@ void FileSet::Sub(int i)
    if(ind>i)
       ind--;
 }
-
-void FileSet::Merge(const FileSet *set)
+FileInfo *FileSet::Borrow(int i)
 {
+   FileInfo *fi=files[i].borrow();
+   Sub(i);
+   return fi;
+}
+
+void FileSet::assert_sorted() const
+{
+   for(int i=0; i<fnum-1; i++)
+      assert(strcmp(files[i]->name,files[i+1]->name)<0);
+}
+
+void FileSet::Merge_insert(const FileSet *set)
+{
+   if(!set)
+      return;
    for(int i=0; i<set->fnum; i++)
    {
       const Ref<FileInfo>& fi=set->files[i];
@@ -134,6 +148,52 @@ void FileSet::Merge(const FileSet *set)
 	 add_before(pos,new FileInfo(*fi));
    }
 }
+void FileSet::Merge(const FileSet *set)
+{
+   assert(!sorted);
+   if(!set || !set->fnum)
+      return;
+
+   // estimate work to be done by Merge_insert
+   int pos = FindGEIndByName(set->files[0]->name);
+   if(fnum-pos < fnum*2/set->fnum) {
+      Merge_insert(set);
+      return;
+   }
+
+   RefArray<FileInfo> new_set;
+   int i=0;
+   int j=0;
+   while(i<set->fnum && j<fnum) {
+      Ref<FileInfo>& fi1=files[j];
+      const Ref<FileInfo>& fi2=set->files[i];
+      int cmp = strcmp(fi1->name,fi2->name);
+      if(cmp==0) {
+	 fi1->Merge(*fi2);
+	 new_set.append(fi1.borrow());
+	 i++; j++;
+      } else if(cmp>0) {
+	 new_set.append(new FileInfo(*fi2));
+	 i++;
+      } else {
+	 new_set.append(fi1.borrow());
+	 j++;
+      }
+   }
+   while(i<set->fnum) {
+      const Ref<FileInfo>& fi2=set->files[i];
+      new_set.append(new FileInfo(*fi2));
+      i++;
+   }
+   if(new_set.count()==0)
+      return;
+   while(j<fnum) {
+      Ref<FileInfo>& fi1=files[j];
+      new_set.append(fi1.borrow());
+      j++;
+   }
+   files.move_here(new_set);
+}
 
 void FileSet::PrependPath(const char *path)
 {
@@ -142,7 +202,7 @@ void FileSet::PrependPath(const char *path)
 }
 
 FileSet::FileSet()
-   : ind(0)
+   : sort_mode(BYNAME), ind(0)
 {
 }
 
@@ -167,44 +227,44 @@ static int (*compare)(const char *s1, const char *s2);
 static int rev_cmp;
 static RefArray<FileInfo> *files_cmp;
 
-static int sort_name(const void *s1, const void *s2)
+static int sort_name(const int *s1, const int *s2)
 {
-   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
-   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
+   const FileInfo *p1=(*files_cmp)[*s1];
+   const FileInfo *p2=(*files_cmp)[*s2];
    return compare(p1->name, p2->name) * rev_cmp;
 }
 
-static int sort_size(const void *s1, const void *s2)
+static int sort_size(const int *s1, const int *s2)
 {
-   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
-   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
+   const FileInfo *p1=(*files_cmp)[*s1];
+   const FileInfo *p2=(*files_cmp)[*s2];
    if(p1->size > p2->size) return -rev_cmp;
    if(p1->size < p2->size) return rev_cmp;
    return 0;
 }
 
-static int sort_dirs(const void *s1, const void *s2)
+static int sort_dirs(const int *s1, const int *s2)
 {
-   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
-   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
+   const FileInfo *p1=(*files_cmp)[*s1];
+   const FileInfo *p2=(*files_cmp)[*s2];
    if((p1->filetype == FileInfo::DIRECTORY) && !(p2->filetype == FileInfo::DIRECTORY)) return -rev_cmp;
    if(!(p1->filetype == FileInfo::DIRECTORY) && (p2->filetype == FileInfo::DIRECTORY)) return rev_cmp;
    return 0;
 }
 
-static int sort_rank(const void *s1, const void *s2)
+static int sort_rank(const int *s1, const int *s2)
 {
-   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
-   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
+   const FileInfo *p1=(*files_cmp)[*s1];
+   const FileInfo *p2=(*files_cmp)[*s2];
    if(p1->GetRank()==p2->GetRank())
       return sort_name(s1,s2);
    return p1->GetRank()<p2->GetRank() ? -rev_cmp : rev_cmp;
 }
 
-static int sort_date(const void *s1, const void *s2)
+static int sort_date(const int *s1, const int *s2)
 {
-   const FileInfo *p1=(*files_cmp)[*static_cast<const int*>(s1)];
-   const FileInfo *p2=(*files_cmp)[*static_cast<const int*>(s2)];
+   const FileInfo *p1=(*files_cmp)[*s1];
+   const FileInfo *p2=(*files_cmp)[*s2];
    if(p1->date==p2->date)
       return sort_name(s1,s2);
    return p1->date>p2->date ? -rev_cmp : rev_cmp;
@@ -223,19 +283,29 @@ void FileSet::Sort(sort_e newsort, bool casefold, bool reverse)
    rev_cmp=(reverse?-1:1);
    files_cmp=&files;
 
+   xmap<bool> dup;
    sorted.truncate();
-   for(int i=0; i<fnum; i++)
+   for(int i=0; i<fnum; i++) {
+      if(newsort==BYNAME_FLAT && sort_mode!=BYNAME_FLAT) {
+	 Ref<FileInfo>& fi=files[i];
+	 fi->longname.set(fi->name);
+	 fi->name.set(basename_ptr(fi->name));
+	 if(dup.exists(fi->name))
+	    continue;
+	 dup.add(fi->name,true);
+      }
       sorted.append(i);
-
-   int *s=sorted.get_non_const();
-   int sz=sorted.get_element_size();
-   switch(newsort) {
-   case BYNAME: qsort(s, fnum, sz, sort_name); break;
-   case BYSIZE: qsort(s, fnum, sz, sort_size); break;
-   case DIRSFIRST: qsort(s, fnum, sz, sort_dirs); break;
-   case BYRANK: qsort(s, fnum, sz, sort_rank); break;
-   case BYDATE: qsort(s, fnum, sz, sort_date); break;
    }
+
+   switch(newsort) {
+   case BYNAME_FLAT: /*fallthrough*/
+   case BYNAME: sorted.qsort(sort_name); break;
+   case BYSIZE: sorted.qsort(sort_size); break;
+   case DIRSFIRST: sorted.qsort(sort_dirs); break;
+   case BYRANK: sorted.qsort(sort_rank); break;
+   case BYDATE: sorted.qsort(sort_date); break;
+   }
+   sort_mode=newsort;
 }
 
 // reverse current sort order
@@ -253,11 +323,21 @@ void FileSet::ReverseSort()
    }
 }
 
-/* Remove the current sort, allowing new entries to be added.
- * (Nothing uses this ... */
+/* Remove the current sort, allowing new entries to be added. */
 void FileSet::Unsort()
 {
    sorted.unset();
+   if(sort_mode==BYNAME_FLAT)
+      UnsortFlat();
+   sort_mode=BYNAME;
+}
+
+void FileSet::UnsortFlat()
+{
+   for(int i=0; i<files.count(); i++) {
+      assert(files[i]->longname!=0);
+      files[i]->name.move_here(files[i]->longname);
+   }
 }
 
 void FileSet::Empty()
@@ -498,11 +578,14 @@ void FileSet::Count(int *d,int *f,int *s,int *o) const
       switch(files[i]->filetype)
       {
       case(FileInfo::DIRECTORY):
-	 if(d) (*d)++; break;
+	 if(d) (*d)++;
+	 break;
       case(FileInfo::NORMAL):
-	 if(f) (*f)++; break;
+	 if(f) (*f)++;
+	 break;
       case(FileInfo::SYMLINK):
-	 if(s) (*s)++; break;
+	 if(s) (*s)++;
+	 break;
       default:
 	 if(o) (*o)++;
       }
@@ -570,7 +653,7 @@ static bool do_exclude_match(const char *prefix,const FileInfo *fi,const Pattern
    return x->MatchExclude(name);
 }
 
-void  FileSet::Exclude(const char *prefix,const PatternSet *x)
+void  FileSet::Exclude(const char *prefix,const PatternSet *x,FileSet *fsx)
 {
    if(!x)
       return;
@@ -578,7 +661,10 @@ void  FileSet::Exclude(const char *prefix,const PatternSet *x)
    {
       if(do_exclude_match(prefix,files[i],x))
       {
-	 Sub(i);
+	 if(fsx)
+	    fsx->Add(Borrow(i));
+	 else
+	    Sub(i);
 	 i--;
       }
    }
@@ -596,7 +682,7 @@ void FileSet::Dump(const char *tag) const
 
 // *** Manipulations with set of local files
 
-void FileSet::LocalUtime(const char *dir,bool only_dirs)
+void FileSet::LocalUtime(const char *dir,bool only_dirs,bool flat)
 {
    for(int i=0; i<fnum; i++)
    {
@@ -610,7 +696,10 @@ void FileSet::LocalUtime(const char *dir,bool only_dirs)
 	 if(only_dirs && file->filetype!=file->DIRECTORY)
 	    continue;
 
-	 const char *local_name=dir_file(dir,file->name);
+	 const char *name=file->name;
+	 if(flat)
+	    name=basename_ptr(name);
+	 const char *local_name=dir_file(dir,name);
 	 struct utimbuf ut;
 	 struct stat st;
 	 ut.actime=ut.modtime=file->date;
@@ -620,7 +709,7 @@ void FileSet::LocalUtime(const char *dir,bool only_dirs)
       }
    }
 }
-void FileSet::LocalChmod(const char *dir,mode_t mask)
+void FileSet::LocalChmod(const char *dir,mode_t mask,bool flat)
 {
    for(int i=0; i<fnum; i++)
    {
@@ -631,7 +720,10 @@ void FileSet::LocalChmod(const char *dir,mode_t mask)
 	 && file->filetype==file->SYMLINK)
 	    continue;
 
-	 const char *local_name=dir_file(dir,file->name);
+	 const char *name=file->name;
+	 if(flat)
+	    name=basename_ptr(name);
+	 const char *local_name=dir_file(dir,name);
 
 	 struct stat st;
 	 mode_t new_mode=file->mode&~mask;
@@ -641,7 +733,7 @@ void FileSet::LocalChmod(const char *dir,mode_t mask)
       }
    }
 }
-void FileSet::LocalChown(const char *dir)
+void FileSet::LocalChown(const char *dir,bool flat)
 {
    for(int i=0; i<fnum; i++)
    {
@@ -655,7 +747,10 @@ void FileSet::LocalChown(const char *dir)
 #define lchown chown
 #endif
 
-	 const char *local_name=dir_file(dir,file->name);
+	 const char *name=file->name;
+	 if(flat)
+	    name=basename_ptr(name);
+	 const char *local_name=dir_file(dir,name);
 
 	 struct stat st;
 
