@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ class Fish : public SSH_Access
    void	 SendMethod();
    void	 SendArrayInfoRequests();
 
-   void Disconnect();
+   void DisconnectLL();
    int IsConnected() const
       {
 	 if(state==DISCONNECTED)
@@ -114,7 +114,7 @@ public:
 
    int Do();
    int Done();
-   int Read(void *,int);
+   int Read(Buffer *,int);
    int Write(const void *,int);
    int StoreStatus();
    int Buffered();
@@ -150,7 +150,7 @@ class FishDirList : public DirList
 
 public:
    FishDirList(Fish *s,ArgV *a)
-      : DirList(s,a), pattern(args->Combine(1)) {}
+      : DirList(s,a), pattern(args->CombineShellQuoted(1)) {}
    const char *Status();
    int Do();
 
