@@ -377,8 +377,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    if(remove_source_files)
 	       args.Append("-E");
 	    args.Append("-O");
-	    args.Append(target_is_local?target_dir
-			:target_session->GetConnectURL());
+	    args.Append(target_is_local?target_dir.get()
+			:target_session->GetConnectURL().get());
 	    args.Append(source_session->GetFileURL(file->name));
 	    xstring_ca cmd(args.CombineQuoted());
 	    fprintf(script,"%s\n",cmd.get());
@@ -758,7 +758,7 @@ void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
 	    ArgV *a=new ArgV("mkdir");
 	    a->Append(target_dir);
 	    mkdirJob *mkj=new mkdirJob(target_session->Clone(),a);
-	    mkj->cmdline.set_allocated(a->Combine());
+	    a->CombineTo(mkj->cmdline);
 	    JobStarted(mkj);
 	 }
       }
@@ -1062,7 +1062,7 @@ int   MirrorJob::Do()
 	    ArgV *a=new ArgV("mkdir");
 	    a->Append(file->name);
 	    mkdirJob *mkj=new mkdirJob(target_session->Clone(),a);
-	    mkj->cmdline.set_allocated(a->Combine());
+	    a->CombineTo(mkj->cmdline);
 	    JobStarted(mkj);
 	    m=MOVED;
 	 }
@@ -1181,7 +1181,7 @@ int   MirrorJob::Do()
 	    args->Append(file->name);
 	    args->seek(1);
 	    rmJob *j=new rmJob(target_session->Clone(),args);
-	    j->cmdline.set_allocated(args->Combine());
+	    args->CombineTo(j->cmdline);
 	    JobStarted(j);
 	    if(file->TypeIs(file->DIRECTORY))
 	    {
@@ -1259,7 +1259,7 @@ int   MirrorJob::Do()
 	    a->seek(1);
 	    ChmodJob *cj=new ChmodJob(target_session->Clone(),
 				 file->mode&~mode_mask,a);
-	    cj->cmdline.set_allocated(a->Combine());
+	    a->CombineTo(cj->cmdline);
 	    if(!verbose_report)
 	       cj->BeQuiet(); // chmod is not supported on all servers; be quiet.
 	    JobStarted(cj);
@@ -1313,7 +1313,7 @@ int   MirrorJob::Do()
 	    args->Append(source_dir);
 	    args->seek(1);
 	    rmJob *j=new rmJob(source_session->Clone(),args);
-	    j->cmdline.set_allocated(args->Combine());
+	    args->CombineTo(j->cmdline);
 	    j->Rmdir();
 	    JobStarted(j);
 	 }
@@ -1380,7 +1380,7 @@ int   MirrorJob::Do()
 	    args->Append(file->name);
 	    args->seek(1);
 	    rmJob *j=new rmJob(source_session->Clone(),args);
-	    j->cmdline.set_allocated(args->Combine());
+	    args->CombineTo(j->cmdline);
 	    JobStarted(j);
 	 }
 	 const char *source_name_rel=dir_file(source_relative_dir,file->name);
@@ -1875,8 +1875,8 @@ CMD(mirror)
    bool  remove_source_files=false;
    bool  remove_source_dirs=false;
    bool	 skip_noaccess=ResMgr::QueryBool("mirror:skip-noaccess",0);
-   int	 parallel=ResMgr::Query("mirror:parallel-transfer-count",0);
-   int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
+   int	 parallel=-1;
+   int	 use_pget=-1;
    bool	 reverse=false;
    bool	 script_only=false;
    bool	 no_empty_dirs=ResMgr::QueryBool("mirror:no-empty-dirs",0);
@@ -2227,6 +2227,23 @@ CMD(mirror)
       return 0;
    }
 
+   if(parallel<0) {
+      int parallel1=ResMgr::Query("mirror:parallel-transfer-count",source_session->GetHostName());
+      int parallel2=ResMgr::Query("mirror:parallel-transfer-count",target_session->GetHostName());
+      if(parallel1>0)
+	 parallel=parallel1;
+      if(parallel2>0 && (parallel<0 || parallel>parallel2))
+	 parallel=parallel2;
+   }
+   if(use_pget<0) {
+      int use_pget1=ResMgr::Query("mirror:use-pget-n",source_session->GetHostName());
+      int use_pget2=ResMgr::Query("mirror:use-pget-n",target_session->GetHostName());
+      if(use_pget1>0)
+	 use_pget=use_pget1;
+      if(use_pget2>0 && (use_pget<0 || use_pget>use_pget2))
+	 use_pget=use_pget2;
+   }
+
    JobRef<MirrorJob> j(new MirrorJob(0,source_session.borrow(),target_session.borrow(),source_dir,target_dir));
    j->SetFlags(flags,1);
    j->SetVerbose(verbose);
